@@ -16,26 +16,26 @@
 # under the License.
 from __future__ import annotations
 
-import argparse
 import contextlib
 import importlib
 import json
 from datetime import datetime
 from io import StringIO
+from multiprocessing import Process, Queue
 from pathlib import Path
-from subprocess import Popen
-from unittest.mock import MagicMock, call, patch
+from unittest import mock
+from unittest.mock import call, patch
 
 import pytest
 import time_machine
-from requests import HTTPError, Response
+from aiohttp import ClientResponseError, RequestInfo
+from yarl import URL
 
 from airflow.cli import cli_parser
-from airflow.executors import executor_loader
 from airflow.providers.common.compat.sdk import timezone
-from airflow.providers.edge3.cli import edge_command
+from airflow.providers.edge3.cli import edge_command, worker as worker_module
 from airflow.providers.edge3.cli.dataclasses import Job
-from airflow.providers.edge3.cli.worker import EdgeWorker
+from airflow.providers.edge3.cli.worker import EdgeWorker, _execution_api_server_url
 from airflow.providers.edge3.models.edge_worker import (
     EdgeWorkerModel,
     EdgeWorkerState,
@@ -46,11 +46,12 @@ from airflow.providers.edge3.worker_api.datamodels import (
     WorkerRegistrationReturn,
     WorkerSetStateReturn,
 )
-from airflow.utils.state import TaskInstanceState
 
 from tests_common.test_utils.config import conf_vars
+from tests_common.test_utils.version_compat import AIRFLOW_V_3_2_PLUS
 
 pytest.importorskip("pydantic", minversion="2.0.0")
+pytestmark = [pytest.mark.asyncio]
 
 
 MOCK_COMMAND = {
@@ -74,7 +75,7 @@ MOCK_COMMAND = {
 }
 
 
-class _MockPopen(Popen):
+class _MockProcess(Process):
     def __init__(self, returncode=None):
         self.generated_returncode = None
 
@@ -87,16 +88,22 @@ class _MockPopen(Popen):
 
 
 class TestEdgeWorker:
-    parser: argparse.ArgumentParser
-
-    @classmethod
-    def setup_class(cls):
-        with conf_vars(
-            {("core", "executor"): "airflow.providers.edge3.executors.edge_executor.EdgeExecutor"}
-        ):
-            importlib.reload(executor_loader)
+    @pytest.fixture(autouse=True)
+    def setup_parser(self):
+        if AIRFLOW_V_3_2_PLUS:
             importlib.reload(cli_parser)
-            cls.parser = cli_parser.get_parser()
+            self.parser = cli_parser.get_parser()
+        else:
+            with patch(
+                "airflow.executors.executor_loader.ExecutorLoader.get_executor_names",
+            ) as mock_get_executor_names:
+                mock_get_executor_names.return_value = [
+                    mock.MagicMock(
+                        name="EdgeExecutor", module_path="airflow.providers.edge3.executors.EdgeExecutor"
+                    )
+                ]
+                importlib.reload(cli_parser)
+                self.parser = cli_parser.get_parser()
 
     @pytest.fixture
     def mock_joblist(self, tmp_path: Path) -> list[Job]:
@@ -114,7 +121,7 @@ class TestEdgeWorker:
                     concurrency_slots=1,
                     command=MOCK_COMMAND,  # type: ignore[arg-type]
                 ),
-                process=_MockPopen(),
+                process=_MockProcess(),
                 logfile=logfile,
                 logsize=0,
             ),
@@ -135,21 +142,6 @@ class TestEdgeWorker:
         )
         return test_edgeworker
 
-    @patch("airflow.providers.edge3.cli.worker.Process")
-    def test_launch_job(self, mock_process, worker_with_job: EdgeWorker):
-        mock_process_instance = MagicMock()
-        mock_process.side_effect = [mock_process_instance]
-
-        edge_job = EdgeWorker.jobs.pop().edge_job
-        with conf_vars({("edge", "api_url"): "https://invalid-api-test-endpoint"}):
-            worker_with_job._launch_job(edge_job)
-
-        assert mock_process.call_count == 1
-        assert mock_process_instance.start.call_count == 1
-
-        assert len(EdgeWorker.jobs) == 1
-        assert EdgeWorker.jobs[0].edge_job == edge_job
-
     @pytest.mark.parametrize(
         ("configs", "expected_url"),
         [
@@ -176,113 +168,149 @@ class TestEdgeWorker:
         expected_url,
     ):
         with conf_vars(configs):
-            EdgeWorker._execution_api_server_url.cache_clear()
-            url = EdgeWorker._execution_api_server_url()
+            _execution_api_server_url.cache_clear()
+            url = _execution_api_server_url()
             assert url == expected_url
 
     @patch("airflow.sdk.execution_time.supervisor.supervise")
-    @patch("airflow.providers.edge3.cli.worker.Process")
-    def test_supervise_launch(
+    @pytest.mark.asyncio
+    async def test_supervise_launch(
         self,
-        mock_process,
         mock_supervise,
         worker_with_job: EdgeWorker,
     ):
-        mock_process_instance = MagicMock()
-        mock_process.side_effect = [mock_process_instance]
+        edge_job = worker_with_job.jobs.pop().edge_job
+        q = mock.MagicMock()
+        result = worker_with_job._run_job_via_supervisor(edge_job.command, q)
 
-        edge_job = EdgeWorker.jobs.pop().edge_job
-        worker_with_job._launch_job(edge_job)
+        assert result == 0
+        q.put.assert_not_called()
 
-        mock_process_callback = mock_process.call_args.kwargs["target"]
-        mock_process_callback(workload=MagicMock(), execution_api_server_url="http://mock-url")
-
-        assert mock_supervise.call_args.kwargs["server"] == "http://mock-url"
-
-    @pytest.mark.parametrize(
-        ("reserve_result", "fetch_result", "expected_calls"),
-        [
-            pytest.param(None, False, (0, 0), id="no_job"),
-            pytest.param(
-                EdgeJobFetched(
-                    dag_id="test",
-                    task_id="test",
-                    run_id="test",
-                    map_index=-1,
-                    try_number=1,
-                    concurrency_slots=1,
-                    command=MOCK_COMMAND,  # type: ignore[arg-type]
-                ),
-                True,
-                (1, 1),
-                id="new_job",
-            ),
-        ],
-    )
-    @patch("airflow.providers.edge3.cli.worker.jobs_fetch")
-    @patch("airflow.providers.edge3.cli.worker.jobs_set_state")
-    @patch("subprocess.Popen")
-    def test_fetch_job(
+    @patch("airflow.sdk.execution_time.supervisor.supervise")
+    @pytest.mark.asyncio
+    async def test_supervise_launch_fail(
         self,
-        mock_popen,
-        mock_set_state,
-        mock_reserve_task,
-        reserve_result,
-        fetch_result,
-        expected_calls,
+        mock_supervise,
         worker_with_job: EdgeWorker,
     ):
-        logfile_path_call_count, set_state_call_count = expected_calls
-        mock_reserve_task.side_effect = [reserve_result]
-        mock_popen.side_effect = ["mock"]
-        with conf_vars({("edge", "api_url"): "https://invalid-api-test-endpoint"}):
-            got_job = worker_with_job.fetch_job()
-        mock_reserve_task.assert_called_once()
-        assert got_job == fetch_result
-        assert mock_set_state.call_count == set_state_call_count
+        mock_supervise.side_effect = Exception("Supervise failed")
+        edge_job = worker_with_job.jobs.pop().edge_job
+        q = mock.MagicMock()
+        result = worker_with_job._run_job_via_supervisor(edge_job.command, q)
 
-    def test_check_running_jobs_running(self, worker_with_job: EdgeWorker):
-        assert worker_with_job.free_concurrency == worker_with_job.concurrency
-        with conf_vars({("edge", "api_url"): "https://invalid-api-test-endpoint"}):
-            worker_with_job.check_running_jobs()
-        assert len(EdgeWorker.jobs) == 1
-        assert (
-            worker_with_job.free_concurrency
-            == worker_with_job.concurrency - EdgeWorker.jobs[0].edge_job.concurrency_slots
-        )
+        assert result == 1
+        q.put.assert_called_once()
 
+    @patch("airflow.providers.edge3.cli.worker.jobs_fetch")
+    @patch("airflow.providers.edge3.cli.worker.EdgeWorker._launch_job", return_value=(Process(), Queue()))
+    @pytest.mark.asyncio
+    async def test_fetch_and_run_job_no_job(
+        self,
+        mock_launch_job,
+        mock_jobs_fetch,
+        worker_with_job: EdgeWorker,
+    ):
+        mock_jobs_fetch.return_value = None
+
+        await worker_with_job.fetch_and_run_job()
+
+        mock_jobs_fetch.assert_called_once()
+        assert len(worker_with_job.jobs) == 1  # no new job added
+        mock_launch_job.assert_not_called()
+
+    @patch("airflow.providers.edge3.cli.worker.jobs_fetch")
+    @patch("airflow.providers.edge3.cli.worker.EdgeWorker._launch_job", return_value=(Process(), Queue()))
     @patch("airflow.providers.edge3.cli.worker.jobs_set_state")
-    def test_check_running_jobs_success(self, mock_set_state, worker_with_job: EdgeWorker):
-        job = EdgeWorker.jobs[0]
-        job.process.generated_returncode = 0  # type: ignore[union-attr]
-        with conf_vars({("edge", "api_url"): "https://invalid-api-test-endpoint"}):
-            worker_with_job.check_running_jobs()
-        assert len(EdgeWorker.jobs) == 0
-        mock_set_state.assert_called_once_with(job.edge_job.key, TaskInstanceState.SUCCESS)
-        assert worker_with_job.free_concurrency == worker_with_job.concurrency
+    @patch("airflow.providers.edge3.cli.worker.EdgeWorker._push_logs_in_chunks")
+    @patch("airflow.providers.edge3.cli.worker.logs_push")
+    @patch.object(Job, "is_running", property(lambda _: False))
+    @patch.object(Job, "is_success", property(lambda _: True))
+    @pytest.mark.asyncio
+    async def test_fetch_and_run_job_one_job(
+        self,
+        mock_logs_push,
+        mock_push_log_chunks,
+        mock_jobs_set_state,
+        mock_launch_job,
+        mock_jobs_fetch,
+        worker_with_job: EdgeWorker,
+    ):
+        mock_jobs_fetch.side_effect = [
+            EdgeJobFetched(
+                dag_id="test",
+                task_id="test",
+                run_id="test",
+                map_index=-1,
+                try_number=1,
+                concurrency_slots=1,
+                command=MOCK_COMMAND,  # type: ignore[arg-type]
+            ),
+            None,
+        ]
+        worker_with_job.concurrency = 1  # only one job at a time
+        assert worker_with_job.free_concurrency == 0
 
+        await worker_with_job.fetch_and_run_job()
+
+        mock_jobs_fetch.assert_called_once()
+        mock_launch_job.assert_called_once()
+        assert mock_jobs_set_state.call_count == 2
+        mock_push_log_chunks.assert_called_once()
+        assert len(worker_with_job.jobs) == 1  # no new job added (was removed at the end...)
+        mock_logs_push.assert_not_called()
+
+    @patch("airflow.providers.edge3.cli.worker.jobs_fetch")
+    @patch("airflow.providers.edge3.cli.worker.EdgeWorker._launch_job", return_value=(Process(), Queue()))
     @patch("airflow.providers.edge3.cli.worker.jobs_set_state")
-    def test_check_running_jobs_failed(self, mock_set_state, worker_with_job: EdgeWorker):
-        job = EdgeWorker.jobs[0]
-        job.process.generated_returncode = 42  # type: ignore[union-attr]
-        with conf_vars({("edge", "api_url"): "https://invalid-api-test-endpoint"}):
-            worker_with_job.check_running_jobs()
-        assert len(EdgeWorker.jobs) == 0
-        mock_set_state.assert_called_once_with(job.edge_job.key, TaskInstanceState.FAILED)
-        assert worker_with_job.free_concurrency == worker_with_job.concurrency
+    @patch("airflow.providers.edge3.cli.worker.EdgeWorker._push_logs_in_chunks")
+    @patch("airflow.providers.edge3.cli.worker.logs_push")
+    @patch.object(Job, "is_running", property(lambda _: False))
+    @patch.object(Job, "is_success", property(lambda _: False))
+    @patch("traceback.format_exception", return_value=[])
+    @pytest.mark.asyncio
+    async def test_fetch_and_run_job_one_job_fail(
+        self,
+        mock_traceback,
+        mock_logs_push,
+        mock_push_log_chunks,
+        mock_jobs_set_state,
+        mock_launch_job,
+        mock_jobs_fetch,
+        worker_with_job: EdgeWorker,
+    ):
+        mock_jobs_fetch.side_effect = [
+            EdgeJobFetched(
+                dag_id="test",
+                task_id="test",
+                run_id="test",
+                map_index=-1,
+                try_number=1,
+                concurrency_slots=1,
+                command=MOCK_COMMAND,  # type: ignore[arg-type]
+            ),
+            None,
+        ]
+        worker_with_job.concurrency = 1  # only one job at a time
+        assert worker_with_job.free_concurrency == 0
+
+        await worker_with_job.fetch_and_run_job()
+
+        mock_jobs_fetch.assert_called_once()
+        mock_launch_job.assert_called_once()
+        assert mock_jobs_set_state.call_count == 2
+        mock_push_log_chunks.assert_called_once()
+        assert len(worker_with_job.jobs) == 1  # no new job added (was removed at the end...)
+        mock_logs_push.assert_called_once()
 
     @time_machine.travel(datetime.now(), tick=False)
     @patch("airflow.providers.edge3.cli.worker.logs_push")
-    def test_check_running_jobs_log_push(self, mock_logs_push, worker_with_job: EdgeWorker):
+    @pytest.mark.asyncio
+    async def test_push_logs_in_chunks(self, mock_logs_push, worker_with_job: EdgeWorker):
         job = EdgeWorker.jobs[0]
         job.logfile.write_text("some log content")
-        with conf_vars(
-            {
-                ("edge", "api_url"): "https://invalid-api-test-endpoint",
-                ("edge", "push_log_chunk_size"): "524288",
-            }
-        ):
-            worker_with_job.check_running_jobs()
+        with conf_vars({("edge", "api_url"): "https://invalid-api-test-endpoint"}):
+            await worker_with_job._push_logs_in_chunks(job)
+
         assert len(EdgeWorker.jobs) == 1
         mock_logs_push.assert_called_once_with(
             task=job.edge_job.key, log_chunk_time=timezone.utcnow(), log_chunk_data="some log content"
@@ -290,18 +318,14 @@ class TestEdgeWorker:
 
     @time_machine.travel(datetime.now(), tick=False)
     @patch("airflow.providers.edge3.cli.worker.logs_push")
-    def test_check_running_jobs_log_push_increment(self, mock_logs_push, worker_with_job: EdgeWorker):
+    @pytest.mark.asyncio
+    async def test_check_running_jobs_log_push_increment(self, mock_logs_push, worker_with_job: EdgeWorker):
         job = EdgeWorker.jobs[0]
         job.logfile.write_text("hello ")
         job.logsize = job.logfile.stat().st_size
         job.logfile.write_text("hello world")
-        with conf_vars(
-            {
-                ("edge", "api_url"): "https://invalid-api-test-endpoint",
-                ("edge", "push_log_chunk_size"): "524288",
-            }
-        ):
-            worker_with_job.check_running_jobs()
+        with conf_vars({("edge", "api_url"): "https://invalid-api-test-endpoint"}):
+            await worker_with_job._push_logs_in_chunks(job)
         assert len(EdgeWorker.jobs) == 1
         mock_logs_push.assert_called_once_with(
             task=job.edge_job.key, log_chunk_time=timezone.utcnow(), log_chunk_data="world"
@@ -309,13 +333,13 @@ class TestEdgeWorker:
 
     @time_machine.travel(datetime.now(), tick=False)
     @patch("airflow.providers.edge3.cli.worker.logs_push")
-    def test_check_running_jobs_log_push_chunks(self, mock_logs_push, worker_with_job: EdgeWorker):
+    @patch.object(worker_module, "push_log_chunk_size", 4)
+    @pytest.mark.asyncio
+    async def test_check_running_jobs_log_push_chunks(self, mock_logs_push, worker_with_job: EdgeWorker):
         job = EdgeWorker.jobs[0]
         job.logfile.write_bytes("log1log2ülog3".encode("latin-1"))
-        with conf_vars(
-            {("edge", "api_url"): "https://invalid-api-test-endpoint", ("edge", "push_log_chunk_size"): "4"}
-        ):
-            worker_with_job.check_running_jobs()
+        with conf_vars({("edge", "api_url"): "https://invalid-api-test-endpoint"}):
+            await worker_with_job._push_logs_in_chunks(job)
         assert len(EdgeWorker.jobs) == 1
         calls = mock_logs_push.call_args_list
         assert len(calls) == 4
@@ -348,7 +372,7 @@ class TestEdgeWorker:
         ],
     )
     @patch("airflow.providers.edge3.cli.worker.worker_set_state")
-    def test_heartbeat(
+    async def test_heartbeat(
         self, mock_set_state, drain, maintenance_mode, jobs, expected_state, worker_with_job: EdgeWorker
     ):
         if not jobs:
@@ -359,7 +383,7 @@ class TestEdgeWorker:
             state=EdgeWorkerState.RUNNING, queues=["queue1", "queue2"]
         )
         with conf_vars({("edge", "api_url"): "https://invalid-api-test-endpoint"}):
-            worker_with_job.heartbeat()
+            await worker_with_job.heartbeat()
         assert mock_set_state.call_args.args[1] == expected_state
         queue_list = worker_with_job.queues or []
         assert len(queue_list) == 2
@@ -367,47 +391,60 @@ class TestEdgeWorker:
         assert "queue2" in (queue_list)
 
     @patch("airflow.providers.edge3.cli.worker.worker_set_state")
-    def test_version_mismatch(self, mock_set_state, worker_with_job):
+    async def test_version_mismatch(self, mock_set_state, worker_with_job):
         mock_set_state.side_effect = EdgeWorkerVersionException("")
-        worker_with_job.heartbeat()
+        await worker_with_job.heartbeat()
         assert worker_with_job.drain
 
+    @pytest.mark.parametrize(
+        "http_error",
+        [
+            pytest.param(404, id="HTTP 404 Not Found"),
+            pytest.param(405, id="HTTP 405 Method Not Allowed"),
+        ],
+    )
     @patch("airflow.providers.edge3.cli.worker.worker_register")
-    def test_start_missing_apiserver(self, mock_register_worker, worker_with_job: EdgeWorker):
-        mock_response = Response()
-        mock_response.status_code = 404
-        mock_register_worker.side_effect = HTTPError(
-            "Something with 404:NOT FOUND means API is not active", response=mock_response
+    async def test_start_missing_apiserver(
+        self, mock_register_worker, http_error, worker_with_job: EdgeWorker
+    ):
+        mock_register_worker.side_effect = ClientResponseError(
+            request_info=RequestInfo(url=URL("mock.com"), method="GET", headers=None),  # type:ignore[arg-type]
+            message=f"Something with {http_error}: Means API is not active",
+            status=http_error,
+            history=(),
         )
         with pytest.raises(SystemExit, match=r"API endpoint is not ready"):
-            worker_with_job.start()
+            await worker_with_job.start()
 
     @patch("airflow.providers.edge3.cli.worker.worker_register")
-    def test_start_server_error(self, mock_register_worker, worker_with_job: EdgeWorker):
-        mock_response = Response()
-        mock_response.status_code = 500
-        mock_register_worker.side_effect = HTTPError(
-            "Something other error not FourhundretFour", response=mock_response
+    async def test_start_server_error(self, mock_register_worker, worker_with_job: EdgeWorker):
+        mock_register_worker.side_effect = ClientResponseError(
+            request_info=RequestInfo(url=URL("mock.com"), method="GET", headers=None),  # type:ignore[arg-type]
+            message="Something other error not FourhundretFour",
+            status=500,
+            history=(),
         )
         with pytest.raises(SystemExit, match=r"Something other"):
-            worker_with_job.start()
+            await worker_with_job.start()
 
     @patch("airflow.providers.edge3.cli.worker.worker_register")
     @patch("airflow.providers.edge3.cli.worker.EdgeWorker.loop")
     @patch("airflow.providers.edge3.cli.worker.worker_set_state")
-    def test_start_and_run_one(self, mock_set_state, mock_loop, mock_register, worker_with_job: EdgeWorker):
+    async def test_start_and_run_one(
+        self, mock_set_state, mock_loop, mock_register, worker_with_job: EdgeWorker
+    ):
         def stop_running():
-            EdgeWorker.drain = True
-            EdgeWorker.jobs = []
+            worker_with_job.drain = True
+            worker_with_job.jobs = []
 
         mock_loop.side_effect = stop_running
         mock_register.side_effect = [WorkerRegistrationReturn(last_update=datetime.now())]
 
-        worker_with_job.start()
+        await worker_with_job.start()
 
         mock_register.assert_called_once()
         mock_loop.assert_called_once()
-        assert mock_set_state.call_count == 2
+        assert mock_set_state.call_count == 1
 
     def test_get_sysinfo(self, worker_with_job: EdgeWorker):
         concurrency = 8
